@@ -1,7 +1,7 @@
 /*
 Draw and interface with our switcher window.
 
-Copyright 2017 Alexander Kulak.
+Copyright 2017-2018 Alexander Kulak.
 This file is part of alttab program.
 
 alttab is free software: you can redistribute it and/or modify
@@ -26,6 +26,7 @@ along with alttab.  If not, see <http://www.gnu.org/licenses/>.
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+//#include <sys/time.h>
 #include "alttab.h"
 #include "util.h"
 extern Globals g;
@@ -36,8 +37,9 @@ extern Window root;
 // PRIVATE
 
 unsigned int tileW, tileH, iconW, iconH;
-int scrNum;
-int scrW, scrH;
+unsigned int visualTileW;
+int lastPressedTile;
+quad scrdim;
 Window uiwin;
 int uiwinW, uiwinH, uiwinX, uiwinY;
 Colormap colormap;
@@ -64,7 +66,7 @@ GC create_gc(int type)
 
 	gc = XCreateGC(dpy, root, valuemask, &values);
 	if (gc < 0) {
-		fprintf(stderr, "can't create GC\n");
+		msg(-1, "can't create GC\n");
 		return 0;
 	}
 	/* allocate foreground and background colors for this GC. */
@@ -88,7 +90,7 @@ GC create_gc(int type)
 				   join_style);
 		break;
 	default:
-		fprintf(stderr, "unknown GC type, not setting colors\n");
+		msg(-1, "unknown GC type, not setting colors\n");
 		break;
 	}
 	/* define the fill style for the GC. to be 'solid filling'. */
@@ -106,7 +108,7 @@ void drawFr(GC gc, int f)
 			       0 + (FRAME_W / 2),
 			       tileW + FRAME_W, tileH + FRAME_W);
 	if (!d) {
-		fprintf(stderr, "can't draw frame\n");
+		msg(-1, "can't draw frame\n");
 	}
 }
 
@@ -126,11 +128,26 @@ void framesRedraw()
 	drawFr(g.gcFrame, g.selNdx);
 }
 
+//
+// given coordinates relative to our window,
+// return the tile number or -1
+//
+int pointedTile(int x, int y)
+{
+    if (x < (FRAME_W / 2)
+            || x > (uiwinW - (FRAME_W / 2))
+            || y < 0
+            || y > uiwinH )
+        return -1;
+    return (x - (FRAME_W / 2)) / visualTileW;
+}
+
 // PUBLIC
 
 //
 // early initialization
 // called once per execution
+// mostly initializes g.*
 // TODO: counterpair for freeing X resources, 
 //   even if called once per execution:
 /*
@@ -144,16 +161,15 @@ if (g.gcFrame) XFreeGC (dpy, g.gcFrame);
 */
 int startupGUItasks()
 {
-
-	scrW = DisplayWidth(dpy, scr);
-	scrH = DisplayHeight(dpy, scr);
-
+// if viewport is not fixed, then initialize vp* at every show
+    if (g.option_vp_mode == VP_SPECIFIC) {
+        g.vp = g.option_vp;
+    }
+    g.has_randr = randrAvailable();
 // colors
 	colormap = DefaultColormap(dpy, scr);
 	visual = DefaultVisual(dpy, scr);
-	if (g.debug > 0) {
-		fprintf(stderr, "early allocating colors\n");
-	}
+    msg(0, "early allocating colors\n");
 	srand(time(NULL));
 	int p;
 	for (p = 0; p < NCOLORS; p++) {
@@ -180,39 +196,34 @@ int startupGUItasks()
 						       r[1] + 0x80);
 					g.color[p].name[7] = '\0';
 				}
-				if (g.debug > 1)
-					fprintf(stderr,
-						"color generated: %s, RAND_MAX=%d\n",
-						g.color[p].name, RAND_MAX);
+                msg(1,
+				  "color generated: %s, RAND_MAX=%d\n",
+				  g.color[p].name, RAND_MAX);
 			}
 			if (!XAllocNamedColor(dpy,
 					      colormap,
 					      g.color[p].name,
 					      &(g.color[p].xcolor),
 					      &(g.color[p].xcolor)))
-				die2("failed to allocate X color: ",
+				die("failed to allocate X color: ",
 				     g.color[p].name);
 			if (!XftColorAllocName
 			    (dpy, visual, colormap, g.color[p].name,
 			     &(g.color[p].xftcolor)))
-				die2("failed to allocate Xft color: ",
+				die("failed to allocate Xft color: ",
 				     g.color[p].name);
 		}
 	}
 
-	if (g.debug > 0) {
-		fprintf(stderr, "early opening font\n");
-	}
+    msg(0, "early opening font\n");
 //fontLabel = XLoadFont (dpy, LABELFONT);  // using Xft instead
-	fontLabel = XftFontOpenName(dpy, scrNum, g.option_font);
+	fontLabel = XftFontOpenName(dpy, scr, g.option_font);
 	if (!fontLabel) {
-		fprintf(stderr, "can't allocate font: %s", g.option_font);
+		msg(-1, "can't allocate font: %s\ncheck installed fontconfig fonts: fc-list\n", g.option_font);
 	}
 // having colors, GC may be built
 // they are required early for addWindow when transforming icon depth
-	if (g.debug > 0) {
-		fprintf(stderr, "early building GCs\n");
-	}
+    msg(0, "early building GCs\n");
 	g.gcDirect = create_gc(0);
 	g.gcReverse = create_gc(1);
 	g.gcFrame = create_gc(2);
@@ -230,45 +241,87 @@ int startupGUItasks()
 //
 int uiShow(bool direction)
 {
-	if (g.debug > 0) {
-		fprintf(stderr, "preparing ui\n");
-	}
-// init X junk early, because depth conversion in addWindow requires GC
-
+    msg(0, "preparing ui\n");
 	g.uiShowHasRun = true;	// begin allocations
+// screen-related stuff is not at startup but here,
+// because screen configuration may get changed at runtime
+// moreover, DisplayWidth/Height aren't changed without
+// reconnecting to X server, that's why root geometry is used
+    XWindowAttributes ra;
+    if (XGetWindowAttributes(dpy, root, &ra) != 0) {
+        scrdim.x = ra.x;
+        scrdim.y = ra.y;
+        scrdim.w = ra.width;
+        scrdim.h = ra.height;
+    } else {
+        msg(-1,
+          "can't get root window attributes, using screen dimensions\n");
+        scrdim.x = scrdim.y = 0;
+    	scrdim.w = DisplayWidth(dpy, scr);
+    	scrdim.h = DisplayHeight(dpy, scr);
+    }
+// caculate viewport.
+#define VPM  g.option_vp_mode
+    switch (VPM) {
+        case VP_SPECIFIC:
+            // initialized at startup instead
+            break;
+        case VP_TOTAL:
+            g.vp = scrdim;
+            break;
+        case VP_FOCUS:
+        case VP_POINTER:
+            if (g.has_randr) {
+                bool multihead;
+                if (! randrGetViewport (&(g.vp), &multihead)) {
+                    msg(0, 
+                      "can't obtain viewport from randr, using default screen\n");
+                    g.vp = scrdim;
+                }
+                if (! multihead) {
+                    msg(0, 
+                      "randr reports single head, using default screen instead\n");
+                    g.vp = scrdim;
+                }
+            } else {
+                msg(0, 
+                  "no randr, using default screen as viewport\n");
+                g.vp = scrdim;
+            }
+            break;
+        default:
+            msg(-1, 
+              "unknown viewport mode, using default screen\n");
+            g.vp = scrdim;
+    }
 
-// GC initialized earlier, now able to init winlist
+    XClassHint class_h = { XCLASSNAME, XCLASS };
+
+// to init winlist, the following must be initialized:
+// GC,
+// g.vp (for SCR_CURRENT)
 
 	g.winlist = NULL;
 	g.maxNdx = 0;
 	if (!initWinlist(direction)) {
-		if (g.debug > 0) {
-			fprintf(stderr,
-				"initWinlist failed, skipping ui initialization\n");
-		}
+        msg(0, "initWinlist failed, skipping ui initialization\n");
 		g.winlist = NULL;
 		g.maxNdx = 0;
 		return 0;
 	}
 	if (!g.winlist) {
-		if (g.debug > 0) {
-			fprintf(stderr,
-				"winlist doesn't exist, skipping ui initialization\n");
-		}
+        msg(0, "winlist doesn't exist, skipping ui initialization\n");
 		return 0;
 	}
 	if (g.maxNdx < 1) {
-		if (g.debug > 0) {
-			fprintf(stderr,
-				"number of windows < 1, skipping ui initialization\n");
-		}
+        msg(0, "number of windows < 1, skipping ui initialization\n");
 		return 0;
 	}
 	if (g.debug > 0) {
-		fprintf(stderr, "got %d windows\n", g.maxNdx);
+		msg(0, "got %d windows\n", g.maxNdx);
 		int i;
 		for (i = 0; i < g.maxNdx; i++) {
-			fprintf(stderr,
+			msg(0,
 				"%d: %lx (lvl %d, icon %lu (%dx%d)): %s\n", i,
 				g.winlist[i].id, g.winlist[i].reclevel,
 				g.winlist[i].icon_drawable, g.winlist[i].icon_w,
@@ -277,17 +330,24 @@ int uiShow(bool direction)
 	}
 // have winlist, now back to uiwin stuff
 // calculate dimensions
-	tileW = g.option_tileW, tileH = g.option_tileH;
+	tileW = g.option_tileW;
+    tileH = g.option_tileH;
 	iconW = g.option_iconW;
 	iconH = g.option_iconH;
 	float rt = 1.0;
-// tiles may be smaller if they don't fit screen
+// for subsequent calculation of width(s), use 'avail_w'
+// instead of g.vp.w, because they don't match for POS_SPECIFIC
+    int avail_w = g.vp.w;
+    if (g.option_positioning == POS_SPECIFIC)
+        avail_w -= g.option_posX;
+// tiles may be smaller if they don't fit viewport
 	uiwinW = (tileW + FRAME_W) * g.maxNdx + FRAME_W;
-	if (uiwinW > scrW) {
-		rt = (float)scrW / (float)uiwinW;
-		uiwinW = scrW;
-		tileW = tileW * rt;
-		tileH = tileH * rt;
+	if (uiwinW > avail_w) {
+        int frames = FRAME_W * g.maxNdx + FRAME_W;
+		rt = ((float)(avail_w - frames)) / ((float)(tileW * g.maxNdx));
+		tileW = (float)tileW * rt;
+		tileH = (float)tileH * rt;
+		uiwinW = tileW * g.maxNdx + frames;
 	}
 // icon may be smaller if it doesn't fit tile
 	if (iconW > tileW) {
@@ -301,11 +361,30 @@ int uiShow(bool direction)
 		iconW = rt * iconW;
 	}
 	uiwinH = tileH + 2 * FRAME_W;
-	uiwinX = (scrW - uiwinW) / 2;
-	uiwinY = (scrH - uiwinH) / 2;
+    if (g.option_positioning == POS_CENTER) {
+    	uiwinX = (g.vp.w - uiwinW) / 2 + g.vp.x;
+    	uiwinY = (g.vp.h - uiwinH) / 2 + g.vp.y;
+    } else {
+        uiwinX = g.option_posX + g.vp.x;
+        uiwinY = g.option_posY + g.vp.y;
+    }
+    visualTileW = (uiwinW - FRAME_W) / g.maxNdx;
 	if (g.debug > 0) {
-		fprintf(stderr, "tile w=%d h=%d win w=%d h=%d\n", tileW, tileH,
-			uiwinW, uiwinH);
+		msg(0, "tile w=%d h=%d\n", tileW, tileH);
+        msg(0, "uiwin %dx%d +%d+%d", uiwinW, uiwinH, uiwinX, uiwinY);
+        if (g.debug > 1) {
+            int nscr, si;
+            Screen *s;
+            nscr = ScreenCount(dpy);
+            msg(0, ", %d screen(s):", nscr);
+            for (si = 0; si < nscr; ++si) {
+                s = ScreenOfDisplay(dpy, si);
+                msg(0, " [%dx%d]", s->width, s->height);
+            }
+            msg(0, ", viewport %dx%d+%d+%d, avail_w %d", 
+                    g.vp.w, g.vp.h, g.vp.x, g.vp.y, avail_w);
+        }
+        msg(0, "\n");
 	}
 // prepare tiles
 
@@ -321,17 +400,14 @@ int uiShow(bool direction)
 		    XFillRectangle(dpy, g.winlist[m].tile, g.gcReverse, 0, 0,
 				   tileW, tileH);
 		if (!fr) {
-			fprintf(stderr, "can't fill tile\n");
+			msg(-1, "can't fill tile\n");
 		}
 		// place icons
 		if (g.winlist[m].icon_drawable) {
 			if (g.winlist[m].icon_w == iconW &&
 			    g.winlist[m].icon_h == iconH) {
 				// direct copy
-				if (g.debug > 1) {
-					fprintf(stderr, "%d: copying icon\n",
-						m);
-				}
+                msg(1, "%d: copying icon\n", m);
 				// prepare special GC to copy icon, with clip mask if icon_mask present
 				unsigned long ic_valuemask = 0;
 				XGCValues ic_values;
@@ -339,8 +415,7 @@ int uiShow(bool direction)
 				    XCreateGC(dpy, root, ic_valuemask,
 					      &ic_values);
 				if (ic_gc < 0) {
-					fprintf(stderr,
-						"can't create GC to draw icon\n");
+					msg(-1, "can't create GC to draw icon\n");
 					return 0;
 				}
 				if (g.winlist[m].icon_mask != 0) {
@@ -355,15 +430,11 @@ int uiShow(bool direction)
 						   g.winlist[m].icon_w, g.winlist[m].icon_h,	// src
 						   0, 0);	// dst
 				if (!or) {
-					fprintf(stderr,
-						"can't copy icon to tile\n");
+					msg(-1, "can't copy icon to tile\n");
 				}
 			} else {
 				// scale
-				if (g.debug > 1) {
-					fprintf(stderr, "%d: scaling icon\n",
-						m);
-				}
+                msg(1, "%d: scaling icon\n", m);
 				int sc = pixmapFit(g.winlist[m].icon_drawable,
 						   g.winlist[m].icon_mask,
 						   g.winlist[m].tile,
@@ -371,15 +442,14 @@ int uiShow(bool direction)
 						   g.winlist[m].icon_h,
 						   iconW, iconH);
 				if (!sc) {
-					fprintf(stderr,
-						"can't scale icon to tile\n");
+                    msg(-1, "can't scale icon to tile\n");
 				}
 			}
 		} else {
 			// draw placeholder or standalone icons from some WM
 			GC gcL = create_gc(0);	// GC for thin line
 			if (!gcL) {
-				fprintf(stderr, "can't create gcL\n");
+				msg(-1, "can't create gcL\n");
 			} else {
 				XSetLineAttributes(dpy, gcL, 1, LineSolid,
 						   CapButt, JoinMiter);
@@ -388,8 +458,7 @@ int uiShow(bool direction)
 				    XDrawRectangle(dpy, g.winlist[m].tile, gcL,
 						   0, 0, iconW, iconH);
 				if (!pr) {
-					fprintf(stderr,
-						"can't draw placeholder\n");
+					msg(-1, "can't draw placeholder\n");
 				}
 				XFreeGC(dpy, gcL);
 			}
@@ -403,30 +472,40 @@ int uiShow(bool direction)
 					  0, (iconH + 5), tileW,
 					  (tileH - iconH - 5));
 			if (dr != 1) {
-				fprintf(stderr, "can't draw label\n");
+				msg(-1, "can't draw label\n");
 			}
 		}
 	}
-	if (g.debug > 0) {
-		fprintf(stderr, "prepared %d tiles\n", m);
-	}
+    msg(0, "prepared %d tiles\n", m);
 	if (fontLabel)
 		XftFontClose(dpy, fontLabel);
 
 // prepare our window
-	uiwin = XCreateSimpleWindow(dpy, root,
-				    uiwinX, uiwinY,
-				    uiwinW, uiwinH,
-				    0, g.color[COLFRAME].xcolor.pixel,
-				    g.color[COLBG].xcolor.pixel);
+	unsigned long valuemask = CWBackPixel | CWBorderPixel | CWOverrideRedirect;
+	XSetWindowAttributes attributes;
+	attributes.background_pixel = g.color[COLBG].xcolor.pixel;
+	attributes.border_pixel = g.color[COLFRAME].xcolor.pixel;
+	attributes.override_redirect = 1;
+	uiwin = XCreateWindow(
+				dpy, root,
+				uiwinX, uiwinY,
+				uiwinW, uiwinH,
+				0, // border_width
+				CopyFromParent, // depth
+				InputOutput, // class
+				CopyFromParent, // visual
+				valuemask, &attributes);
 	if (uiwin <= 0)
 		die("can't create window");
-	if (g.debug > 0) {
-		fprintf(stderr, "our window is %lu\n", uiwin);
-	}
+    msg(0, "our window is 0x%lx\n", uiwin);
+
 // set properties of our window
-	XStoreName(dpy, uiwin, XWINNAME);
-	XSelectInput(dpy, uiwin, ExposureMask | KeyPressMask | KeyReleaseMask);
+    XStoreName(dpy, uiwin, XWINNAME);
+    XSetClassHint(dpy, uiwin, &class_h);
+// warning: this overwrites any previous value.
+// note: x_setCommonPropertiesForAnyWindow does the same thing for any window
+	XSelectInput(dpy, uiwin, ExposureMask | KeyPressMask | KeyReleaseMask 
+        | ButtonPressMask | ButtonReleaseMask);
 // set window type so that WM will hopefully not resize it
 // before mapping: https://specifications.freedesktop.org/wm-spec/1.3/ar01s05.html
 	Atom at = XInternAtom(dpy, "ATOM", True);
@@ -434,11 +513,55 @@ int uiShow(bool direction)
 	Atom td = XInternAtom(dpy, "_NET_WM_WINDOW_TYPE_DIALOG", False);
 	if (at && wt && td)
 		XChangeProperty(dpy, uiwin, wt, at, 32, PropModeReplace,
-				(unsigned char *)(&td), 1);
+            (unsigned char *)(&td), 1);
+// disable appearance in taskbar
+    Atom st = XInternAtom(dpy, "_NET_WM_STATE", True);
+    Atom sk = XInternAtom(dpy, "_NET_WM_STATE_SKIP_TASKBAR", True); // there is also PAGER
+    if (at && st && sk)
+        XChangeProperty(dpy, uiwin, st, at, 32, PropModeReplace,
+            (unsigned char *)(&sk), 1);
 // xmonad ignores _NET_WM_WINDOW_TYPE_DIALOG but obeys WM_TRANSIENT_FOR
 	XSetTransientForHint(dpy, uiwin, uiwin);
+// disable window title and borders. works in xfwm4.
+#define PROP_MOTIF_WM_HINTS_ELEMENTS 5
+#define MWM_HINTS_DECORATIONS (1L << 1)
+    struct {
+        unsigned long flags;
+        unsigned long functions;
+        unsigned long decorations;
+        long inputMode;
+        unsigned long status;
+    } hints = { MWM_HINTS_DECORATIONS, 0, 0, };
+    Atom ma = XInternAtom(dpy, "_MOTIF_WM_HINTS", False);
+    if (ma) {
+        XChangeProperty(dpy, uiwin, ma, ma, 32, PropModeReplace,
+            (unsigned char *)&hints, PROP_MOTIF_WM_HINTS_ELEMENTS);
+    }
+
+    XMapWindow(dpy, uiwin);
+
+    // positioning and size hints.
+    // centering required in JWM.
+    // should really perform centering when 
+    //  viewport == wm screen. how would we know the latter?
+    long sflags;
+    sflags = USPosition|USSize|PPosition|PSize|PMinSize|PMaxSize|PBaseSize;
+    // gravity: https://tronche.com/gui/x/xlib/window/attributes/gravity.html
+    if (g.option_positioning != POS_NONE)
+        sflags |= PWinGravity;
+    XSizeHints uiwinSizeHints = { sflags,
+        uiwinX, uiwinY, // obsoleted
+        uiwinW, uiwinH, // obsoleted
+        uiwinW, uiwinH,
+        uiwinW, uiwinH,
+        0, 0,
+        {0,0}, {0,0},
+        uiwinW, uiwinH,
+        (g.option_positioning == POS_CENTER 
+         //&& g.option_vp_mode != VP_SPECIFIC) ? CenterGravity : ForgetGravity };
+         && g.option_vp_mode != VP_SPECIFIC) ? CenterGravity : StaticGravity };
+    XSetWMNormalHints(dpy, uiwin, &uiwinSizeHints);
 
-	XMapWindow(dpy, uiwin);
 	return 1;
 }
 
@@ -448,25 +571,43 @@ int uiShow(bool direction)
 //
 void uiExpose()
 {
-	if (g.debug > 0) {
-		fprintf(stderr, "expose ui\n");
-	}
+    msg(0, "expose ui\n");
+// if WM moved uiwin, here is the place
+// where we first see 'bad' absolute coordinates.
+// try to correct them.
+    quad uwq;
+    if (get_absolute_coordinates(uiwin, &uwq)) {
+// debug for #54
+        msg(1, "attr abs at expose: %dx%d +%d+%d\n",
+          uwq.w, uwq.h, uwq.x, uwq.y);
+        int xdiff = uwq.x - uiwinX;
+        int ydiff = uwq.y - uiwinY;
+        if (abs(xdiff) > FRAME_W / 2 || 
+          abs(ydiff) > FRAME_W / 2) {
+            msg(1, 
+              "WM moved uiwin too far, trying to correct\n");
+            XMoveWindow(dpy, uiwin, uiwinX, uiwinY);
+        }
+        if (uwq.w != uiwinW || uwq.h != uiwinH) {
+            // WM resized our window, like
+            // floating_maximum_size in #54.
+            // there is little can be done here,
+            // so just complain.
+            msg(-1, 
+              "switcher window resized, expect bugs. Please configure WM to not interfere with alttab window size, for example, disable 'floating_maximum_size' in i3\n");
+        }
+    }
 // icons
 	int j;
 	for (j = 0; j < g.maxNdx; j++) {
 		if (g.winlist[j].tile) {
-			if (g.debug > 1) {
-				fprintf(stderr, "copying tile %d to canvas\n",
-					j);
-			}
+            msg(1, "copying tile %d to canvas\n", j);
 			//XSync (dpy, false);
 			int r = XCopyArea(dpy, g.winlist[j].tile, uiwin,
 					  g.gcDirect, 0, 0, tileW, tileH,	// src
 					  j * (tileW + FRAME_W) + FRAME_W, FRAME_W);	// dst
 			//XSync (dpy, false);
-			if (g.debug > 1) {
-				fprintf(stderr, "XCopyArea returned %d\n", r);
-			}
+            msg(1, "XCopyArea returned %d\n", r);
 		}
 	}
 // frame
@@ -478,21 +619,27 @@ void uiExpose()
 //
 int uiHide()
 {
-	if (g.winlist) {
-		if (g.debug > 0) {
-			fprintf(stderr, "changing window focus to %lu\n",
-				g.winlist[g.selNdx].id);
-		}
-		setFocus(g.selNdx);	// before winlist destruction!
-	}
-	if (g.debug > 0) {
-		fprintf(stderr, "destroying our window and tiles\n");
-	}
+    // order is important: to set focus in Metacity,
+    // our window must be destroyed first
 	if (uiwin) {
+        msg(0, "destroying our window\n");
 		XUnmapWindow(dpy, uiwin);
 		XDestroyWindow(dpy, uiwin);
 		uiwin = 0;
 	}
+	if (g.winlist) {
+        msg(0, "changing focus to 0x%lx\n",
+		  g.winlist[g.selNdx].id);
+        /*
+        // save the switch moment for detecting
+        // subsequent false focus event from WM
+        gettimeofday(&(g.last.tv), NULL);
+        g.last.prev = g.winlist[g.startNdx].id;
+        g.last.to = g.winlist[g.selNdx].id;
+        */
+		setFocus(g.selNdx);	// before winlist destruction!
+	}
+    msg(0, "destroying tiles\n");
 	int y;
 	for (y = 0; y < g.maxNdx; y++) {
 		if (g.winlist && g.winlist[y].tile) {
@@ -517,9 +664,7 @@ int uiNextWindow()
 	g.selNdx++;
 	if (g.selNdx >= g.maxNdx)
 		g.selNdx = 0;
-	if (g.debug > 0) {
-		fprintf(stderr, "item %d\n", g.selNdx);
-	}
+    msg(0, "item %d\n", g.selNdx);
 	framesRedraw();
 	return 1;
 }
@@ -534,9 +679,60 @@ int uiPrevWindow()
 	g.selNdx--;
 	if (g.selNdx < 0)
 		g.selNdx = g.maxNdx - 1;
-	if (g.debug > 0) {
-		fprintf(stderr, "item %d\n", g.selNdx);
-	}
+    msg(0, "item %d\n", g.selNdx);
 	framesRedraw();
 	return 1;
 }
+
+//
+// select item in g.winlist
+//
+int uiSelectWindow(int ndx)
+{
+	if (!uiwin)
+		return 0;	// kb events may trigger it even when no window drawn yet
+    if (ndx < 0 || ndx >= g.maxNdx) {
+        return 0; }
+	g.selNdx = ndx;
+    msg(0, "item %d\n", g.selNdx);
+	framesRedraw();
+	return 1;
+}
+
+//
+// mouse press/release handler
+//
+void uiButtonEvent(XButtonEvent e)
+{
+    if (!uiwin)
+        return;
+    if (e.type == ButtonPress) {
+        switch (e.button) {
+            case 1:
+                lastPressedTile = pointedTile (e.x, e.y);
+                if (lastPressedTile != -1)
+                    uiSelectWindow (lastPressedTile);
+                break;
+            case 4:
+                uiPrevWindow();
+                break;
+            case 5:
+                uiNextWindow();
+                break;
+        }
+    }
+    if (e.type == ButtonRelease && e.button == 1) {
+        if (lastPressedTile != -1 
+                && lastPressedTile == pointedTile (e.x, e.y))
+            uiHide();
+    }
+}
+
+//
+// our window
+//
+Window getUiwin()
+{
+    return uiwin;
+}
+
