@@ -1,7 +1,7 @@
 /*
 util.c definitions.
 
-Copyright 2017 Alexander Kulak.
+Copyright 2017-2018 Alexander Kulak.
 This file is part of alttab program.
 
 alttab is free software: you can redistribute it and/or modify
@@ -27,8 +27,21 @@ along with alttab.  If not, see <http://www.gnu.org/licenses/>.
 #include <X11/Xutil.h>
 #include <X11/Xft/Xft.h>
 #include <X11/extensions/Xrender.h>
+#include <X11/Xresource.h>
+#include <stdarg.h>
+#include <stdio.h>
 
 #define MAXPROPLEN  4096
+#define ERRLEN      2048
+
+#ifndef COMTYPES
+#define COMTYPES
+typedef struct {
+    int w; int h;
+    int x; int y;
+} quad;
+#define MAXNAMESZ   256
+#endif
 
 XErrorEvent *ee_ignored;
 bool ee_complain;
@@ -37,7 +50,7 @@ unsigned int getOffendingModifiersMask();
 int changeKeygrab(Window win, bool grab, KeyCode keycode,
 		  unsigned int modmask, unsigned int ignored_modmask);
 int zeroErrorHandler(Display * dpy_our, XErrorEvent * theEvent);
-void setSelectInput(Window win, int reg);
+//void setSelectInput(Window win, int reg);
 
 int execAndReadStdout(char *exe, char *args[], char *buf, int bufsize);
 
@@ -64,4 +77,13 @@ Bool predproc_true(Display * display, XEvent * event, char *arg);
 char *get_x_property(Window win, Atom prop_type, char *prop_name,
 		     unsigned long *prop_size);
 
+bool rectangles_cross(quad a, quad b);
+bool get_absolute_coordinates(Window w, quad *q);
+
+void remove_arg(int *argc, char **argv, int argn);
+char* xresource_load_string(XrmDatabase *db, const char *appname, char *name);
+int xresource_load_int(XrmDatabase *db, const char *appname, char *name, unsigned int *ret);
+int ksym_option_to_keycode(XrmDatabase *db, const char *appname, const char *name, char **errmsg);
+unsigned int keycode_to_modmask(KeyCode kc);
+
 #endif
