@@ -1,7 +1,7 @@
 /*
 Interface with foreign windows in raw X11.
 
-Copyright 2017 Alexander Kulak.
+Copyright 2017-2018 Alexander Kulak.
 This file is part of alttab program.
 
 alttab is free software: you can redistribute it and/or modify
@@ -61,7 +61,7 @@ Window x_get_leader(Window win)
 // PUBLIC
 
 //
-// set winlist,maxNdx,startNdx recursively using raw Xlib
+// set winlist,maxNdx recursively using raw Xlib
 // first call should be (win=root, reclevel=0)
 //
 int x_initWindowsInfoRecursive(Window win, int reclevel)
@@ -70,36 +70,50 @@ int x_initWindowsInfoRecursive(Window win, int reclevel)
 	Window root, parent;
 	Window *children;
 	unsigned int nchildren, i;
-//Window leader;
+//    Window leader;
 	XWindowAttributes wa;
+    char* winname;
 
 // check if window is "leader" or no prop, skip otherwise
 // caveat: in rp, gvim leader has no file name and icon
 // this doesn't work in raw X: leader may be not viewable.
+// doesn't work in twm either: qutebrowser has meaningless leader.
 /*
-leader = x_get_leader (win);
-if (g.debug>1) {fprintf (stderr, "win: %x leader: %x\n", win, leader);}
+    leader = 0;
+    if (g.option_wm == WM_TWM) {
+        leader = x_get_leader (win);
+        msg(1, "win: 0x%lx leader: 0x%lx\n", win, leader);
+    }
 */
-// add viewable only
+// in non-twm, add viewable only
 // caveat: in rp, skips anything except of visible window
 // probably add an option for this in WMs too?
-	XGetWindowAttributes(dpy, win, &wa);
+    wa.map_state = 0;
+    if (g.option_wm != WM_TWM)
+    	XGetWindowAttributes(dpy, win, &wa);
+
+// in twm-like, add only windows with a name
+    winname = NULL;
+    if (g.option_wm == WM_TWM) {
+        winname = get_x_property(win, XA_STRING, "WM_NAME", NULL);
+    }
 
 // insert detailed window data in window list
-//if ( ((!leader) || leader==win)  &&
-//if ( (leader==win)  &&
-	if (wa.map_state == IsViewable && reclevel != 0) {
-		addWindowInfo(win, reclevel, 0, NULL);
+    if ( (g.option_wm == WM_TWM || wa.map_state == IsViewable) 
+            && reclevel != 0 
+            && (g.option_wm != WM_TWM || winname != NULL)
+//            && (g.option_wm != WM_TWM || leader == win)
+            && ! common_skipWindow(win, DESKTOP_UNKNOWN, DESKTOP_UNKNOWN)
+            ) {
+        addWindowInfo(win, reclevel, 0, DESKTOP_UNKNOWN, winname);
 	}
 // skip children if max recursion level reached
-	if (g.option_max_reclevel != -1 && reclevel >= g.option_max_reclevel)
-		return 1;
+    if (g.option_max_reclevel != -1 && reclevel >= g.option_max_reclevel)
+        return 1;
 
 // recursion
 	if (XQueryTree(dpy, win, &root, &parent, &children, &nchildren) == 0) {
-		if (g.debug > 0) {
-			fprintf(stderr, "can't get window tree for %lu\n", win);
-		}
+        msg(0, "can't get window tree for 0x%lx\n", win);
 		return 0;
 	}
 	for (i = 0; i < nchildren; ++i) {
@@ -109,9 +123,6 @@ if (g.debug>1) {fprintf (stderr, "win: %x leader: %x\n", win, leader);}
 	if (nchildren > 0 && children) {
 		XFree(children);
 	}
-	if (reclevel == 0) {
-		g.startNdx = 0;
-	}
 	return 1;
 }
 
@@ -140,3 +151,27 @@ int x_setFocus(int wndx)
 
 	return 1;
 }
+
+//
+// this is where alttab is supposed to set properties or
+// register interest in event for ANY foreign window encountered.
+// warning: this is called only on addition to sortlist.
+//
+void x_setCommonPropertiesForAnyWindow(Window win)
+{
+    long evmask = 0;
+    // root and our window are treated elsewhere
+    if (win == root || win == getUiwin())
+        return;
+    // for delete notification
+    evmask |= StructureNotifyMask;
+    // for focusIn notification
+    if (g.option_wm != WM_EWMH) {
+        msg(0, "using direct focus tracking for 0x%lx\n", win);
+        evmask |= FocusChangeMask;
+    }
+    // warning: this overwrites previous value
+    if (evmask != 0)
+        XSelectInput(dpy, win, evmask);
+}
+
